#include <gtk/gtk.h>
#include <math.h>

int width = 320;
int height = 240;
int gap = height / 20;
int r = gap;
double visual_length = height - 2 * gap - r;

double phi0 = -0.5;
double omega0 = 0.0;
double t0 = 0.0;
double g = 9.81;
double l = 1.0;
double dt = 0.02;

double t = t0;
double phi = phi0;
double omega = omega0;

gboolean draw (GtkWidget *widget, cairo_t *c, gpointer data)
{
  GdkRGBA blue = { 0.0, 0.5, 1.0, 1.0 };

  int x = width / 2 + visual_length * sin (phi);
  int y = gap + visual_length * cos (phi);

  gdk_cairo_set_source_rgba (c, &blue);
  cairo_move_to (c, width / 2, gap);
  cairo_line_to (c, x, y);
  cairo_stroke (c);
  cairo_arc (c, x, y, r, 0, 2 * G_PI);
  cairo_fill (c);

  return FALSE;  /* TRUE to stop other handlers from being invoked for the event.
                    FALSE to propagate the event further. */
}

gboolean timer (GtkWidget *widget)
{
  t += dt;
  phi += omega * dt;
  omega += - dt * g / l * sin (phi);

  gtk_widget_queue_draw_area (widget, 0, 0, width, height);
  g_timeout_add (50, (GSourceFunc) timer, widget);
  return FALSE;
}

int main (int argc, char **argv)
{
  gtk_init (&argc, &argv);

  GtkWidget *window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_show (window);
  gtk_window_set_title (GTK_WINDOW (window), "Hello");
  g_signal_connect (window, "destroy", G_CALLBACK (gtk_main_quit), NULL);

  GtkWidget *drawing_area = gtk_drawing_area_new ();
  gtk_widget_show (drawing_area);
  gtk_container_add (GTK_CONTAINER (window), drawing_area);
  gtk_widget_set_size_request (drawing_area, width, height);
  g_signal_connect (drawing_area, "draw", G_CALLBACK (draw), NULL);

  g_timeout_add (50, (GSourceFunc) timer, drawing_area);

  gtk_main ();
  return 0;
}
